using System;
using System.Runtime.InteropServices;

namespace Cerebrum.Ganglion.Ensemble
{
	/// <summary>
	/// Summary description for ControllerImplementation.
	/// </summary>
	internal class ControllerImplementation
	{
		/*[DllImport("kernel32.dll", CharSet = CharSet.Auto)]
		private static extern IntPtr GetModuleHandle(String lpModuleName);
		[DllImport("kernel32.dll", CharSet = CharSet.Auto)]
		private static extern bool FreeLibrary(IntPtr hLibModule);*/
		
		private static object m_SyncRoot;


		static ControllerImplementation()
		{
			m_SyncRoot = new object();
		}

		internal static ControllerImplementation m_Instance;
		public static ControllerImplementation Instance
		{
			get
			{
				if(m_Instance==null)
				{
					lock(m_SyncRoot)
					{
						if(m_Instance==null)
						{
							m_Instance = new ControllerImplementation();
						}
					}
				}
				return m_Instance;
			}
		}

		internal ControllerImplementation()
		{
			m_Application = null;
			m_Databases = null;
			System.AppDomain.CurrentDomain.DomainUnload += new EventHandler(CurrentDomain_DomainUnload);
		}

		private Cerebrum.Ganglion.Ensemble.Application m_Application;
		private System.Collections.Hashtable m_Databases;

		private void CurrentDomain_DomainUnload(object sender, EventArgs e)
		{
			Shutdown();
		}

		public void Initialize()
		{
			if(m_Databases==null)
			{
				lock(m_SyncRoot)
				{
					if(m_Databases==null)
					{
						m_Databases = new System.Collections.Hashtable();
					}
				}
			}
			if(m_Application==null)
			{
				lock(m_SyncRoot)
				{
					if(m_Application==null)
					{
						m_Application = new Cerebrum.Ganglion.Ensemble.Application();
						m_Application.Initialize();
					}
				}
			}
		}

		public void Shutdown()
		{
			if(m_Databases!=null)
			{
				lock(m_SyncRoot)
				{
					if(m_Databases!=null)
					{
						foreach(DatabaseController c in m_Databases.Values)
						{
							if(c!=null)
								c.Dispose();
						}
						m_Databases = null;
					}
				}
			}
			if(m_Application!=null)
			{
				lock(m_SyncRoot)
				{
					if(m_Application!=null)
					{
						/*if (m_Application.MasterContext != null)
						{
							m_Application.MasterContext.Workspace.Activator = null;
						}*/
						m_Application.Shutdown();
						m_Application = null;
					}
				}
				/*IntPtr hc = GetModuleHandle("Cerebrum.Runtime.dll");
				FreeLibrary(hc);*/
			}
			Cerebrum.Runtime.Environment.Current.Dispose();
			System.GC.Collect();
			System.GC.WaitForPendingFinalizers();
		}

		public Cerebrum.Ganglion.Ensemble.Application Application
		{
			get
			{
				return this.m_Application;
			}
		}
		public DatabaseController GetDatabaseController(string name, bool autostart)
		{
			lock(m_SyncRoot)
			{
				DatabaseController c = m_Databases==null?null:(DatabaseController)m_Databases[name];
				if(c==null)
				{
					this.Initialize();
					using(Cerebrum.IConnector table = m_Application.MasterContext.GetTable("Domains"))
					{
						using(Cerebrum.Data.TableView view = (table.Component as Cerebrum.Reflection.TableDescriptor).GetTableView())
						{
							using(Cerebrum.IConnector domdesc = view.FindItemViewByObjectHandle(Cerebrum.Management.Utilites.FindHandle(view, view.GetItemProperties(null)["Name"], name)).GetComposite())
							{
								c = new DatabaseController(name, domdesc);
							}
						}
					}
					m_Databases[name] = c;
				}
				if(autostart)
				{
					c.Initialize();
				}
				return c;
			}
		}

		public static object SyncRoot
		{
			get
			{
				return m_SyncRoot;
			}
		}

	}
}
