using System;

namespace Cerebrum.Ganglion.Ensemble
{
	/// <summary>
	/// Summary description for SyncronizedComposite.
	/// </summary>
	public class SyncronizedComposite : System.MarshalByRefObject, Cerebrum.IComposite
	{
		private Cerebrum.IComposite m_Composite;
		public SyncronizedComposite(Cerebrum.IComposite composite)
		{
			m_Composite = composite;
		}
		#region IComposite Members

		public Cerebrum.ObjectHandle GroundHandle
		{
			get
			{
				lock(ControllerImplementation.SyncRoot)
				{
					return m_Composite.GroundHandle;
				}
			}
		}

		public Cerebrum.ObjectHandle LiquidHandle
		{
			get
			{
				lock(ControllerImplementation.SyncRoot)
				{
					return m_Composite.LiquidHandle;
				}
			}
		}

		public Cerebrum.ObjectHandle TypeIdHandle
		{
			get
			{
				lock(ControllerImplementation.SyncRoot)
				{
					return m_Composite.TypeIdHandle;
				}
			}
		}

		public Cerebrum.ObjectHandle KindIdHandle
		{
			get
			{
				lock(ControllerImplementation.SyncRoot)
				{
					return m_Composite.KindIdHandle;
				}
			}
		}

		public Cerebrum.IConnector GetPrecedent()
		{
			lock(ControllerImplementation.SyncRoot)
			{
				return new SyncronizedConnector(m_Composite.GetPrecedent());
			}
		}

		public Cerebrum.ObjectHandle PlasmaHandle
		{
			get
			{
				lock(ControllerImplementation.SyncRoot)
				{
					return m_Composite.PlasmaHandle;
				}
			}
		}

		#endregion

		#region IConnector Members

		public object Component
		{
			get
			{
				lock(ControllerImplementation.SyncRoot)
				{
					return m_Composite.Component;
				}
			}
			set
			{
				lock(ControllerImplementation.SyncRoot)
				{
					m_Composite.Component = value;
				}
			}
		}

		public Cerebrum.IWorkspace Workspace
		{
			get
			{
				lock(ControllerImplementation.SyncRoot)
				{
					return new SyncronizedWorkspace(m_Composite.Workspace);
				}
			}
		}

		#endregion

		#region IDisposable Members

		public event System.EventHandler Disposing
		{
			add
			{
			}
			remove
			{
			}
		}

		#endregion

		#region IDisposable Members

		public void Dispose()
		{
			lock(ControllerImplementation.SyncRoot)
			{
				m_Composite.Dispose();
			}
		}

		#endregion
	}
}
