using System;

namespace Cerebrum.Ganglion.Ensemble
{
	/// <summary>
	/// Summary description for SyncronizedWorkspace.
	/// </summary>
	public class SyncronizedWorkspace : System.MarshalByRefObject, Cerebrum.IWorkspace, Cerebrum.IConnector
	{
		private Cerebrum.IWorkspace m_Workspace;

		public SyncronizedWorkspace(Cerebrum.IWorkspace workspace)
		{
			m_Workspace = workspace;
		}
		#region IConnector Members

		public object Component
		{
			get
			{
				lock(ControllerImplementation.SyncRoot)
				{
					return m_Workspace.Component;
				}
			}
			set
			{
				lock(ControllerImplementation.SyncRoot)
				{
					m_Workspace.Component = value;
				}
			}
		}

		public Cerebrum.IWorkspace Workspace
		{
			get
			{
				lock(ControllerImplementation.SyncRoot)
				{
					return new SyncronizedWorkspace(m_Workspace.Workspace);
				}
			}
		}

		#endregion

		#region IDisposable Members

		public void Dispose()
		{
			// TODO:  Add SyncronizedWorkspace.Dispose implementation
		}

		#endregion

		#region IContainer Members

		public void EngageConnector(Cerebrum.ObjectHandle plasma, Cerebrum.IConnector connector)
		{
			lock(ControllerImplementation.SyncRoot)
			{
				m_Workspace.EngageConnector(plasma, connector);
			}
		}

		public Cerebrum.IComposite AttachConnector(Cerebrum.ObjectHandle plasma)
		{
			lock(ControllerImplementation.SyncRoot)
			{
				return new SyncronizedComposite(m_Workspace.AttachConnector(plasma));
			}
		}

		public void RemoveConnector(Cerebrum.ObjectHandle plasma)
		{
			lock(ControllerImplementation.SyncRoot)
			{
				m_Workspace.RemoveConnector(plasma);
			}
		}

		public Cerebrum.IComposite CreateConnector(Cerebrum.ObjectHandle plasma, Cerebrum.ObjectHandle typeID)
		{
			lock(ControllerImplementation.SyncRoot)
			{
				return new SyncronizedComposite(m_Workspace.CreateConnector(plasma, typeID));
			}
		}

		#endregion

		#region IDisposable Members

		public event System.EventHandler Disposing
		{
			add
			{
			}
			remove
			{
			}
		}

		#endregion

		#region IWorkspace Members

		public Cerebrum.ObjectHandle NextSequence()
		{
			// TODO:  Add SyncronizedWorkspace.NextSequence implementation
			return new Cerebrum.ObjectHandle ();
		}

		public Cerebrum.IMilestones Milestones
		{
			get
			{
				// TODO:  Add SyncronizedWorkspace.Milestones getter implementation
				return null;
			}
		}

		public string Name
		{
			get
			{
				// TODO:  Add SyncronizedWorkspace.Name getter implementation
				return null;
			}
		}

		public Cerebrum.IActivator Activator
		{
			get
			{
				return null;
			}
			set
			{
				// TODO:  Add SyncronizedWorkspace.Activator setter implementation
			}
		}

		public Cerebrum.IFormatter Formatter
		{
			get
			{
				return null;
			}
		}

		public Cerebrum.ObjectHandle CurrSequence()
		{
			// TODO:  Add SyncronizedWorkspace.CurrSequence implementation
			return new Cerebrum.ObjectHandle ();
		}

		public IContainer GetSector()
		{
			throw new Exception("The method or operation is not implemented.");
		}

		#endregion
	}
}
