using System;
using System.Text;

namespace Cerebrum.Ganglion.Indexing
{
	public class Controller : Cerebrum.Integrator.GenericComponent
	{
		private Cerebrum.ObjectHandle[] CreateText(string text, bool isreadonly)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;
			Cerebrum.ObjectHandle root = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Components", "Name", "IndexRoot");
			Cerebrum.IComposite connector = null;
			try
			{
				if (Cerebrum.ObjectHandle.Null.Equals(root))
				{
					connector = this.CreateRoot();

					root = connector.PlasmaHandle;
					using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "Components"))
					{
						int i = view.Add(root, connector);
						view[i]["Name"] = "IndexRoot";
						Cerebrum.Management.Utilites.RefreshHandle(domainContext, "Components", "Name", view[i].ObjectHandle);
					}

				}
				else
				{
					connector = domainContext.AttachConnector(root);
				}

				Cerebrum.Ganglion.Indexing.ILineralTreeNode node = connector.Component as Cerebrum.Ganglion.Indexing.ILineralTreeNode;
				return node.GenerateDependents(connector, "\0" + text, isreadonly);
			}
			finally
			{
				if (connector != null)
				{
					connector.Dispose();
					connector = null;
				}
			}
		}

		private string ObtainText(Cerebrum.ObjectHandle nodeHandle)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;
			using (Cerebrum.IConnector connector = domainContext.AttachConnector(nodeHandle))
			{
				Cerebrum.Ganglion.Indexing.ILineralTreeNode node = connector.Component as Cerebrum.Ganglion.Indexing.ILineralTreeNode;
				string[] res = node.RestoreStrings();
				return res[0].Substring(1);
			}
		}

		private Cerebrum.IComposite CreateRoot()
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;
			Cerebrum.ObjectHandle h0;
			//  ID   
			h0 = domainContext.Workspace.NextSequence();
			//    
			Cerebrum.IComposite connector = domainContext.Workspace.CreateConnector(h0, Cerebrum.Ganglion.Indexing.Specialized.Concepts.SymbolNeuronType(domainContext));

			//      ILineralTreeNodeEx
			//       AddPrecedent      ILineralTreeNode
			ILineralTreeNodeEx node = connector.Component as ILineralTreeNodeEx;
			if (node != null)
			{
				//     
				node.Symbol = '\0';
			}

			return connector;
		}

		public void CreateIndex(Cerebrum.ObjectHandle ringId, string text, Cerebrum.ObjectHandle target)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;

			Cerebrum.ObjectHandle[] trace = this.CreateText(text, false);

			using (Cerebrum.IContainer container = (Cerebrum.IContainer)domainContext.AttachConnector(trace[trace.Length - 1]))
			{
				Cerebrum.IContainer container2 = null;
				try
				{
					container2 = (Cerebrum.IContainer)container.AttachConnector(ringId);
					if(container2==null)
					{
						((Cerebrum.Runtime.NativeWarden)container).Newobj(ringId, Cerebrum.Runtime.KernelObjectKindId.Warden);
						container2 = (Cerebrum.IContainer)container.AttachConnector(ringId);
					}
					Cerebrum.Integrator.Utilites.EngageConnector(container2, target);
				}
				finally
				{
					if (container2 != null)
					{
						container2.Dispose();
						container2 = null;
					}
				}
			}
		}

		public bool RemoveIndex(Cerebrum.ObjectHandle ringId, string text, Cerebrum.ObjectHandle target)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;

			Cerebrum.ObjectHandle[] trace = this.CreateText(text, true);
			if (trace.Length != (text.Length + 1))
			{
				return false;
			}

			using (Cerebrum.IContainer container = (Cerebrum.IContainer)domainContext.AttachConnector(trace[trace.Length - 1]))
			{
				Cerebrum.IContainer container2 = null;
				try
				{
					container2 = (Cerebrum.IContainer)container.AttachConnector(ringId);
					if (container2 != null)
					{
						container2.RemoveConnector(target);
						return true;
					}
				}
				finally
				{
					if (container2 != null)
					{
						container2.Dispose();
						container2 = null;
					}
				}
			}
			return false;
		}

		public Cerebrum.ObjectHandle[] ObtainIndex(Cerebrum.ObjectHandle ringId, string text)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;
			Cerebrum.ObjectHandle[] trace = this.CreateText(text, true);
			if (trace.Length != (text.Length + 1))
			{
				return null;
			}

			using (Cerebrum.IComposite connector = domainContext.AttachConnector(trace[trace.Length - 1]))
			{
				FastSymbolNeuron node = connector.Component as FastSymbolNeuron;
				using (Cerebrum.IContainer container = node.GetAttributeContainer(ringId) as Cerebrum.IContainer)
				{
					System.Collections.ArrayList list = new System.Collections.ArrayList();
					foreach (System.Collections.DictionaryEntry de in (System.Collections.IEnumerable)container)
					{
						list.Add(de.Key);
					}
					return (Cerebrum.ObjectHandle[])list.ToArray(typeof(Cerebrum.ObjectHandle));
				}
			}
		}
		/*private void btnRestoreText_Click(object sender, System.EventArgs e)
		{
			System.DateTime start = System.DateTime.Now;

			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
			{
				if (saveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
				{
					string text;
					using (System.IO.StreamReader sr = new System.IO.StreamReader(openFileDialog1.FileName, System.Text.Encoding.Default))
					{
						text = sr.ReadToEnd();
					}
					string[] codes = text.Split(' ', ',', '.', '\r', '\n');
					foreach (string code in codes)
					{
						if (code.Trim().Length < 1) continue;
						Cerebrum.ObjectHandle h = Cerebrum.ObjectHandle.Parse(code);
						using (Cerebrum.IConnector connector = domainContext.AttachConnector(h))
						{
							Cerebrum.Samples.Objects01.ILineralTreeNode node = connector.Component as Cerebrum.Samples.Objects01.ILineralTreeNode;
							string[] res = node.RestoreStrings();
							sb.Append(res[0].Substring(1));
							sb.Append("\r\n");
						}
					}
					using (System.IO.Stream fs = saveFileDialog1.OpenFile())
					{
						System.IO.StreamWriter sw = new System.IO.StreamWriter(fs);
						sw.Write(sb.ToString());
						sw.Close();
					}
				}
			}
			System.TimeSpan elapsed = System.DateTime.Now - start;
			lblMessage.Text = elapsed.TotalMilliseconds.ToString();
		}

		private void btnFindAllLeafs_Click(object sender, System.EventArgs e)
		{
			System.DateTime start = System.DateTime.Now;

			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			System.Collections.ArrayList ar = new ArrayList();
			if (saveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
			{
				using (Cerebrum.IConnector connector = view[0].GetComposite())
				{
					ILineralTreeNodeEx node = connector.Component as ILineralTreeNodeEx;
					node.FindAllLeafs(ar);
				}

				foreach (string s in ar)
				{
					sb.Append(s.Substring(1));
					sb.Append("\r\n");
				}

				using (System.IO.Stream fs = saveFileDialog1.OpenFile())
				{
					System.IO.StreamWriter sw = new System.IO.StreamWriter(fs);
					sw.Write(sb.ToString());
					sw.Close();
				}
			}
			System.TimeSpan elapsed = System.DateTime.Now - start;
			lblMessage.Text = elapsed.TotalMilliseconds.ToString();
		}*/
	}
}
