using System;

namespace Cerebrum.Ganglion.Remoting
{
	/// <summary>
	/// Summary description for IApplication.
	/// </summary>
	public interface IEnsembleController
	{
		void Initialize();
		void Shutdown();
		IDatabaseController GetDatabaseController(string name, bool autostart);
		object SyncRoot {get;}
	}

	public interface IDatabaseController
	{
		void Initialize();
		void Shutdown();
		Cerebrum.ObjectHandle InsertRecord(string tableName, string []columns, object []values);
		object InvokeRecord(string tableName, Cerebrum.ObjectHandle handle, string method, object[] parameters);
		bool UpdateRecord(string tableName, Cerebrum.ObjectHandle handle, string []columns, object []values);
		bool EngageRecord(string tableName, Cerebrum.ObjectHandle handle);
		bool DeleteRecord(string tableName, Cerebrum.ObjectHandle handle);
		object [] SelectRecord(string tableName, Cerebrum.ObjectHandle handle, string []columns);
		ObjectHandle ResolveHandle(string tableName, string fieldName, object value);
	}
	public interface IRemoteDatabase
	{
		void Shutdown(string databaseName);
		Cerebrum.ObjectHandle InsertRecord(string databaseName, string tableName, string []columns, object []values);
		bool UpdateRecord(string databaseName, string tableName, Cerebrum.ObjectHandle handle, string []columns, object []values);
		object InvokeRecord(string databaseName, string tableName, Cerebrum.ObjectHandle handle, string method, object[] parameters);
		bool EngageRecord(string databaseName, string tableName, Cerebrum.ObjectHandle handle);
		bool DeleteRecord(string databaseName, string tableName, Cerebrum.ObjectHandle handle);
		object [] SelectRecord(string databaseName, string tableName, Cerebrum.ObjectHandle handle, string []columns);
		ObjectHandle ResolveHandle(string databaseName, string tableName, string fieldName, object value);
		TableDef QueryTable(string databaseName, string tableName, string[] columns, int from, int count, object query);
		TableDef QueryRecord(string dbName, string tableName, string fieldName, ObjectHandle handle, string[] columns, int from, int count, object query);
		object InvokeComponent(string databaseName, /*string tableName, */string componentName, string method, object[] parameters);
		void Impersonate(System.Security.Principal.IPrincipal principal);
	}

	[Serializable]
	public struct TupleDef
	{
		public TupleDef(object[] values)
		{
			this.Values = values;
		}
		public object[] Values;
	}

	[Serializable]
	public struct FieldDef
	{
		public FieldDef(string name, System.Type type)
		{
			this.Name = name;
			this.Type = type;
		}
		public string Name;
		public System.Type Type;
	}
	public enum OrderDirection
	{
		Ascending,
		Descending
	}
	[Serializable]
	public class OrderDef
	{
		public OrderDef(string name, OrderDirection direction)
		{
			this.Name = name;
			this.Direction = direction;
		}
		public string Name;
		public OrderDirection Direction;
	}
	[Serializable]
	public class QueryDef
	{
		public QueryDef(OrderDef[] orderBy)
		{
			this.OrderBy = orderBy;
		}

		public OrderDef[] OrderBy;
	}
	[Serializable]
	public class TableDef
	{
		public FieldDef[] Fields;
		public TupleDef[] Tuples;
		public int Count;

		public int GetOrdinal(string name)
		{
			for (int i = 0; i < this.Fields.Length; i++)
			{
				if (name.Equals(((FieldDef)this.Fields[i]).Name))
				{
					return i;
				}
			}
			return -1;
		}

		public object GetValue(int r, string name)
		{
			return this.Tuples[r].Values[this.GetOrdinal(name)];
		}

		public object GetValue(int r, int c)
		{
			return this.Tuples[r].Values[c];
		}
	}
}
