using System;

namespace Cerebrum.Collections.Specialized
{
	/// <summary>
	/// Summary description for HandleMap.
	/// </summary>
	[Serializable()]
	public class HandleObjectDictionary
	{
		private object [] m_Array;

		public HandleObjectDictionary()
		{
			m_Array = new object [1024];
		}

		/// <summary>
		///       ObjectHandle.
		/// </summary>
		public object this[Cerebrum.ObjectHandle ObjectHandle]
		{
			set
			{
				if (value != null)
				{
					int v = ObjectHandle.ToInt32();
					int section3 = (v >> 22) & 0x03FF;
					int section2 = (v >> 11) & 0x07FF;
					int section1 = v & 0x07FF;

					object[] s3;
					object[] s2;

					s3 = (object[])m_Array[section3];
					if (s3 == null)
					{
						s3 = new object[2048];
						m_Array[section3] = s3;
					}

					s2 = (object[])s3[section2];
					if (s2 == null)
					{
						s2 = new object[2048];
						s3[section2] = s2;
					}

					s2[section1] = value;
				}
				else
				{
					this.Remove(ObjectHandle);
				}
			}
			get
			{
				int v = ObjectHandle.ToInt32();
				int section3 = (v >> 22) & 0x03FF;
				int section2 = (v >> 11) & 0x07FF;
				int section1 = v & 0x07FF;

				object[] s3;
				object[] s2;

				if ((s3 = (object[])m_Array[section3]) == null)
				{
					return null;
				}
				else if ((s2 = (object[])s3[section2]) == null)
				{
					return null;
				}
				return s2[section1];
			}
		}

		/// <summary>
		///   .
		/// </summary>
		/// <param name="ObjectHandle">   .</param>
		public void Remove(Cerebrum.ObjectHandle ObjectHandle)
		{
			int v = ObjectHandle.ToInt32();
			int section3 = (v >> 22) & 0x03FF;
			int section2 = (v >> 11) & 0x07FF;
			int section1 = v & 0x07FF;

			object[] s3;
			object[] s2;

			if ((s3 = (object[])m_Array[section3]) == null)
			{
				return;
			}
			else if ((s2 = (object[])s3[section2]) == null)
			{
				return;
			}

			if (s2[section1] != null)
			{
				s2[section1] = null;
				if (this.CalcChildrenCount(s2) < 1)
				{
					s3[section2] = null;
					if (this.CalcChildrenCount(s3) < 1)
					{
						m_Array[section3] = null;
					}
				}
			}
		}

		private int CalcChildrenCount(object[] array)
		{
			int nCount = 0;

			for (int i = 0; i < array.Length; i++)
			{
				if (array[i] != null)
				{
					nCount++;
				}
			}
			return nCount;
		}

	}
}

