using System;
using System.Text;

namespace Cerebrum.Ganglion.WebForum
{
	public class Controller : Cerebrum.Integrator.GenericComponent
	{
		public Cerebrum.ObjectHandle CreateWebMessage(Cerebrum.ObjectHandle threadHandle, string author, string subject, string text, string userhost)
		{
			Cerebrum.Security.CerebrumPrincipal principal = System.Threading.Thread.CurrentPrincipal as Cerebrum.Security.CerebrumPrincipal;

			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;
			Cerebrum.ObjectHandle hindxId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Components", "Name", "Cerebrum.Ganglion.Indexing.Controller");
			Cerebrum.ObjectHandle hringId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Attributes", "Name", "DependentInstances");

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "WebMessages"))
			{
				Cerebrum.Data.ComponentItemView item = view.AddNew();
				using (Cerebrum.IConnector connector = item.GetComposite())
				{
					Cerebrum.Ganglion.WebForum.Reflection.WebMessage msg = connector.Component as Cerebrum.Ganglion.WebForum.Reflection.WebMessage;
					msg.MessageAuthor = author;
					msg.MessageSubject = subject;
					msg.MessageText = text;
					msg.UserHostAddress = userhost;
					msg.CreatedDateTime = System.DateTime.Now;
					if (principal != null)
					{
						msg.SecurityOwner = (principal.Identity as Cerebrum.Security.CerebrumIdentity).PrincipalHandle;
					}
				}
				System.ComponentModel.PropertyDescriptor pdRep = item.GetProperties()["UniqueIdentity"];
				ReIndexReplicationInternal(domainContext, hindxId, hringId, pdRep, item);

				if (threadHandle != Cerebrum.ObjectHandle.Null)
				{
					Cerebrum.Data.ComponentItemView parent = view.CreateItemViewByObjectHandle(threadHandle);
					System.ComponentModel.PropertyDescriptor pdView = view.GetItemProperties(null)["DependentMessages"];

					object v = pdView.GetValue(parent);
					if(v is Cerebrum.Data.SimpleView)
						using (Cerebrum.Data.SimpleView view2 = (Cerebrum.Data.SimpleView)v)
						{
							using (Cerebrum.IComposite composite = item.GetComposite())
							{
								view2.Add(item.ObjectHandle, composite);
							}
							using (Cerebrum.Data.SimpleView view3 = (Cerebrum.Data.SimpleView)item["PrecedentMessages"])
							{
								using (Cerebrum.IComposite cp = parent.GetComposite())
								{
									view3.Add(parent.ObjectHandle, cp);
								}
							}
						}
				}
				return item.ObjectHandle;
			}
		}

		public void DeleteWebMessage(Cerebrum.ObjectHandle recordHandle)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;
			Cerebrum.ObjectHandle hindxId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Components", "Name", "Cerebrum.Ganglion.Indexing.Controller");
			Cerebrum.ObjectHandle hringId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Attributes", "Name", "DependentInstances");

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "WebMessages"))
			{
				Cerebrum.Data.ComponentItemView recordItem = view.CreateItemViewByObjectHandle(recordHandle);
				string uniqueIdentity = Convert.ToString(recordItem["UniqueIdentity"]);
				Cerebrum.Ganglion.Ensemble.Tools.InvokeRecord(domainContext, "Components", hindxId, "RemoveIndex", new object[] { hringId, uniqueIdentity, recordHandle });

				object v2;

				v2 = recordItem["PrecedentMessages"];
				if (v2 is Cerebrum.Data.SimpleView)
					using (Cerebrum.Data.SimpleView view2 = (Cerebrum.Data.SimpleView)v2)
					{
						foreach (Cerebrum.Data.ComponentItemView threadItem in view2)
						{
							object v3 = threadItem["DependentMessages"];
							if (v3 is Cerebrum.Data.SimpleView)
								using (Cerebrum.Data.SimpleView view3 = (Cerebrum.Data.SimpleView)v3)
								{
									view3.Remove(view3.CreateItemViewByObjectHandle(recordHandle));
								}

						}
					}
				v2 = recordItem["DependentMessages"];
				if (v2 is Cerebrum.Data.SimpleView)
					using (Cerebrum.Data.SimpleView view2 = (Cerebrum.Data.SimpleView)v2)
					{
						foreach (Cerebrum.Data.ComponentItemView threadItem in view2)
						{
							object v3 = threadItem["PrecedentMessages"];
							if (v3 is Cerebrum.Data.SimpleView)
								using (Cerebrum.Data.SimpleView view3 = (Cerebrum.Data.SimpleView)v3)
								{
									view3.Remove(view3.CreateItemViewByObjectHandle(recordHandle));
								}

						}
					}
				view.Remove(recordItem);
			}
			domainContext.Workspace.RemoveConnector(recordHandle);
		}

		public void EngageWebMessage(Cerebrum.ObjectHandle threadHandle, Cerebrum.ObjectHandle recordHandle)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "WebMessages"))
			{
				object v;
				Cerebrum.Data.ComponentItemView threadItem = view.CreateItemViewByObjectHandle(threadHandle);
				Cerebrum.Data.ComponentItemView recordItem = view.CreateItemViewByObjectHandle(recordHandle);

				v = threadItem["DependentMessages"];
				if (v is Cerebrum.Data.SimpleView)
					using (Cerebrum.Data.SimpleView view2 = (Cerebrum.Data.SimpleView)v)
					{
						using(Cerebrum.IComposite c3 = recordItem.GetComposite())
						{
							view2.Add(recordHandle, c3);
						}
					}

				v = recordItem["PrecedentMessages"];
				if (v is Cerebrum.Data.SimpleView)
					using (Cerebrum.Data.SimpleView view2 = (Cerebrum.Data.SimpleView)v)
					{
						using (Cerebrum.IComposite c3 = threadItem.GetComposite())
						{
							view2.Add(threadHandle, c3);
						}
					}
			}
		}

		public void RemoveWebMessage(Cerebrum.ObjectHandle threadHandle, Cerebrum.ObjectHandle recordHandle)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "WebMessages"))
			{
				object v;
				Cerebrum.Data.ComponentItemView threadItem = view.CreateItemViewByObjectHandle(threadHandle);
				Cerebrum.Data.ComponentItemView recordItem = view.CreateItemViewByObjectHandle(recordHandle);

				v = threadItem["DependentMessages"];
				if (v is Cerebrum.Data.SimpleView)
					using (Cerebrum.Data.SimpleView view2 = (Cerebrum.Data.SimpleView)v)
					{
						view2.Remove(view2.CreateItemViewByObjectHandle(recordHandle));
					}

				v = recordItem["PrecedentMessages"];
				if (v is Cerebrum.Data.SimpleView)
					using (Cerebrum.Data.SimpleView view2 = (Cerebrum.Data.SimpleView)v)
					{
						view2.Remove(view2.CreateItemViewByObjectHandle(threadHandle));
					}
			}
		}

		public Cerebrum.WebForum.Remoting.WebMessagesResult SelectWebMessages(Cerebrum.ObjectHandle threadHandle, string[] fields, string sortMode, string pageMode, int pageSize, int currPage, Cerebrum.ObjectHandle recordHandle)
		{
			int tmpPageSize;

			Cerebrum.Ganglion.Remoting.IRemoteDatabase ctl = new Cerebrum.Ganglion.Ensemble.RemoteDatabase();

			if (pageSize < 0)
			{
				pageSize = 10;
			}
			tmpPageSize = pageSize;
			if (currPage < 0)
			{
				if (recordHandle != Cerebrum.ObjectHandle.Null)
				{
					currPage = 0;
					tmpPageSize = -1;
				}
				else
				{
					if (pageMode == "r")
					{
						sortMode = "a";
						if (threadHandle != Cerebrum.ObjectHandle.Null)
						{
							Cerebrum.Ganglion.Remoting.TableDef qr = ctl.QueryRecord(Cerebrum.WebForum.Remoting.Specialized.KnownNames.WebForumDatabaseName, "WebMessages", "DependentMessages", threadHandle, new string[] { "ObjectHandle" }, -1, 0, null);
							if (qr != null)
							{
								int pages = (qr.Count + pageSize - 1) / pageSize;
								currPage = pages - 1;
							}
							else
							{
								currPage = 0;
							}
						}
					}
					else
					{
						sortMode = "d";
						currPage = 0;
					}
				}
			}

			if (currPage < 0)
			{
				currPage = 0;
			}
			Cerebrum.Ganglion.Remoting.TableDef queryResult;
			Cerebrum.Ganglion.Remoting.OrderDirection orderDirection = (sortMode != "d") ? Cerebrum.Ganglion.Remoting.OrderDirection.Ascending : Cerebrum.Ganglion.Remoting.OrderDirection.Descending;
			Cerebrum.Ganglion.Remoting.OrderDef orderDef = new Cerebrum.Ganglion.Remoting.OrderDef("CreatedDateTime", orderDirection);
			Cerebrum.Ganglion.Remoting.QueryDef queryDef = new Cerebrum.Ganglion.Remoting.QueryDef(new Cerebrum.Ganglion.Remoting.OrderDef[] { orderDef });
			if (threadHandle == Cerebrum.ObjectHandle.Null)
			{
				queryResult = ctl.QueryTable(Cerebrum.WebForum.Remoting.Specialized.KnownNames.WebForumDatabaseName, "WebMessages", fields, currPage * tmpPageSize, tmpPageSize, queryDef);
			}
			else
			{
				queryResult = ctl.QueryRecord(Cerebrum.WebForum.Remoting.Specialized.KnownNames.WebForumDatabaseName, "WebMessages", "DependentMessages", threadHandle, fields, currPage * tmpPageSize, tmpPageSize, queryDef);
			}
			if (recordHandle != Cerebrum.ObjectHandle.Null)
			{
				Cerebrum.Ganglion.Remoting.TupleDef[] tuples = queryResult.Tuples;
				int xObjectHandle = queryResult.GetOrdinal("ObjectHandle");
				int ordinal = -1;
				for (int i = 0; i < tuples.Length; i++)
				{
					if (recordHandle.Equals(tuples[i].Values[xObjectHandle]))
					{
						ordinal = i;
						break;
					}
				}
				if (ordinal < 0)
				{
					if (currPage < 0)
					{
						currPage = 0;
					}
				}
				else
				{
					currPage = ordinal / pageSize;
				}
				System.Collections.ArrayList list = new System.Collections.ArrayList();
				for (int i = currPage * pageSize; i < tuples.Length && i < ((currPage + 1) * pageSize); i++)
				{
					list.Add(tuples[i]);
				}
				queryResult.Tuples = (Cerebrum.Ganglion.Remoting.TupleDef[])list.ToArray(typeof(Cerebrum.Ganglion.Remoting.TupleDef));
			}
			int xSecurityOwner = queryResult.GetOrdinal("SecurityOwner");
			Cerebrum.Collections.Specialized.HandleObjectDictionary principals = new Cerebrum.Collections.Specialized.HandleObjectDictionary();
			if(queryResult.Tuples!=null)
				for (int i = 0; i < queryResult.Tuples.Length; i++)
				{
					object oh = queryResult.Tuples[i].Values[xSecurityOwner];
					if (oh != null && !Convert.IsDBNull(oh))
					{
						Cerebrum.ObjectHandle ownerHandle = (Cerebrum.ObjectHandle)oh;
						if (principals[ownerHandle] == null)
						{
							principals[ownerHandle] = ctl.SelectRecord(Cerebrum.WebForum.Remoting.Specialized.KnownNames.WebForumDatabaseName, "Principals", ownerHandle, new string[] { "UniqueIdentity", "DisplayName" });
						}
					}
				}
			Cerebrum.WebForum.Remoting.WebMessagesResult wmr = new Cerebrum.WebForum.Remoting.WebMessagesResult();
			wmr.QueryResult = queryResult;
			wmr.PageSize = pageSize;
			wmr.CurrentPage = currPage;
			wmr.Principals = principals;
			return wmr;
		}

		public Cerebrum.ObjectHandle CreatePrincipal(string loginName, string password, string fullName, string description, string email, string homePageURL)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;
			Cerebrum.ObjectHandle hindxId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Components", "Name", "Cerebrum.Ganglion.Indexing.Controller");
			Cerebrum.ObjectHandle hringId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Attributes", "Name", "DependentInstances");

			Cerebrum.ObjectHandle huserId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Principals", "Name", loginName);
			if (huserId != Cerebrum.ObjectHandle.Null)
			{
				throw new System.Exception("User with Name = '" + loginName + "' is already exists");
			}

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "Principals"))
			{
				Cerebrum.Data.ComponentItemView item = view.AddNew();
				using (Cerebrum.IConnector connector = item.GetComposite())
				{
					Cerebrum.Management.Security.Reflection.PrincipalDescriptor prd = connector.Component as Cerebrum.Management.Security.Reflection.PrincipalDescriptor;
					prd.Name = loginName;
					prd.Password = password;
					prd.DisplayName = fullName;
					prd.Description = description;
					prd.EmailAddress = email;
					prd.HomePageURL = homePageURL;
					//prd.Disabled = false;
					//prd.Internal = false;
					//prd.CreatedDateTime = System.DateTime.Now;
				}

				System.ComponentModel.PropertyDescriptor pdUID = item.GetProperties()["UniqueIdentity"];
				ReIndexReplicationInternal(domainContext, hindxId, hringId, pdUID, item);

				Cerebrum.Management.Utilites.RefreshHandle(domainContext, "Principals", "Name", item.ObjectHandle);

				return item.ObjectHandle;
			}
		}

		public void DeletePrincipal(Cerebrum.ObjectHandle recordHandle)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;
			Cerebrum.ObjectHandle hindxId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Components", "Name", "Cerebrum.Ganglion.Indexing.Controller");
			Cerebrum.ObjectHandle hringId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Attributes", "Name", "DependentInstances");

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "Principals"))
			{
				Cerebrum.Data.ComponentItemView recordItem = view.CreateItemViewByObjectHandle(recordHandle);
				string uniqueIdentity = Convert.ToString(recordItem["UniqueIdentity"]);
				Cerebrum.Ganglion.Ensemble.Tools.InvokeRecord(domainContext, "Components", hindxId, "RemoveIndex", new object[] { hringId, uniqueIdentity, recordHandle });

				view.Remove(recordItem);
			}
			domainContext.Workspace.RemoveConnector(recordHandle);
		}

		public Cerebrum.Security.CerebrumPrincipal Authenticate(string userName, string password)
		{
			return Cerebrum.Management.Utilites.Authenticate(this.DomainContext, userName, password);
		}

		public Cerebrum.Security.CerebrumPrincipal LoadPrincipal(Cerebrum.ObjectHandle principalHandle)
		{
			return Cerebrum.Management.Utilites.LoadPrincipal(this.DomainContext, principalHandle, true);
		}

		public string[] RewritePath(string resourceIdentity, string argumentIdentity)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;

			Cerebrum.ObjectHandle hindxId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Components", "Name", "Cerebrum.Ganglion.Indexing.Controller");
			Cerebrum.ObjectHandle hringId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Attributes", "Name", "DependentInstances");

			Cerebrum.ObjectHandle[] hs1 = (Cerebrum.ObjectHandle[])Cerebrum.Ganglion.Ensemble.Tools.InvokeRecord(domainContext, "Components", hindxId, "ObtainIndex", new object[] { hringId, resourceIdentity });
			Cerebrum.ObjectHandle[] hs2 = null;

			if (hs1 != null && hs1.Length > 0)
			{
				if (argumentIdentity!=null && argumentIdentity!=string.Empty)
				{
					hs2 = (Cerebrum.ObjectHandle[])Cerebrum.Ganglion.Ensemble.Tools.InvokeRecord(domainContext, "Components", hindxId, "ObtainIndex", new object[] { hringId, argumentIdentity });
				}

				string p = string.Empty;
				Cerebrum.ObjectHandle h = hs1[0];
				Cerebrum.ObjectHandle v = Cerebrum.ObjectHandle.Null;
				if (hs2 != null && hs2.Length > 0)
				{
					v = hs2[0];
				}
				using (Cerebrum.IComposite connector = domainContext.AttachConnector(h))
				{
					object component = connector.Component;
					if (component is Cerebrum.Management.Security.Reflection.PrincipalDescriptor)
					{
						if (v != Cerebrum.ObjectHandle.Null)
						{
							p = "&w=" + v.ToString();
						}
						return new string[] { "~/UserProfile.aspx", "u=r&c=v&h=" + Convert.ToString(h) + p };
					}
					else if (component is Cerebrum.Management.Security.Reflection.WorkgroupDescriptor)
					{
						if (v != Cerebrum.ObjectHandle.Null)
						{
							p = "&v=" + v.ToString();
						}
						return new string[] { "~/RoleProfile.aspx", "u=w&c=v&h=" + Convert.ToString(h) + p };
					}
					else if (component is Cerebrum.Ganglion.WebForum.Reflection.WebMessage)
					{
						if (v != Cerebrum.ObjectHandle.Null)
						{
							p = "&h=" + v.ToString();
						}
						return new string[] { "~/Record.aspx", "u=r&c=v&t=" + Convert.ToString(h) + p };
					}
				}
			}
			return null;
		}

		public Cerebrum.Ganglion.Remoting.TableDef SelectWorkgroups(Cerebrum.ObjectHandle principalHandle, Cerebrum.Ganglion.Remoting.QueryDef queryDef)
		{
			Cerebrum.Ganglion.Remoting.IRemoteDatabase ctl = new Cerebrum.Ganglion.Ensemble.RemoteDatabase();
			Cerebrum.Ganglion.Remoting.TableDef tdf = ctl.QueryTable(Cerebrum.WebForum.Remoting.Specialized.KnownNames.WebForumDatabaseName, "Workgroups", new string[] { "ObjectHandle", "UniqueIdentity", "Name", "DisplayName", "Description" }, -1, -1, /*this.CurrentPage * this.PageSize, this.PageSize,*/ queryDef);
			int xObjectHandle = tdf.GetOrdinal("ObjectHandle");
			using (Cerebrum.IComposite connector = this.DomainContext.AttachConnector(principalHandle))
			{
				Cerebrum.Management.Security.Reflection.PrincipalDescriptor prd = connector.Component as Cerebrum.Management.Security.Reflection.PrincipalDescriptor;
				using (Cerebrum.Runtime.NativeWarden workgroupsVector = prd.GetSelectedWorkgroupsVector())
				{
					System.Collections.ArrayList fields = new System.Collections.ArrayList(tdf.Fields);
					fields.Add(new Cerebrum.Ganglion.Remoting.FieldDef("Checked", typeof(bool)));
					tdf.Fields = (Cerebrum.Ganglion.Remoting.FieldDef[])fields.ToArray(typeof(Cerebrum.Ganglion.Remoting.FieldDef));
					for (int i = 0; i < tdf.Tuples.Length; i++)
					{
						Cerebrum.Ganglion.Remoting.TupleDef tuple = tdf.Tuples[i];
						System.Collections.ArrayList values = new System.Collections.ArrayList(tuple.Values);
						if (workgroupsVector == null)
						{
							values.Add(false);
						}
						else
						{
							values.Add(workgroupsVector.HasMap((Cerebrum.ObjectHandle)tuple.Values[xObjectHandle]));
						}
						tuple.Values = values.ToArray();
						tdf.Tuples[i] = tuple;
					}
				}
			}
			return tdf;
		}
		public void UpdateWorkgroups(Cerebrum.ObjectHandle principalHandle, Cerebrum.Ganglion.Remoting.TableDef tableDef)
		{
			int xObjectHandle = tableDef.GetOrdinal("ObjectHandle");
			int xChecked = tableDef.GetOrdinal("Checked");
			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(this.DomainContext, "Principals"))
			{
				Cerebrum.Data.ComponentItemView item = view.CreateItemViewByObjectHandle(principalHandle);
				System.ComponentModel.PropertyDescriptor pdSW = view.GetItemProperties(null)["SelectedWorkgroups"];
				using (System.IDisposable dspwrk = (System.IDisposable)pdSW.GetValue(item))
				{
					Cerebrum.Data.SimpleView list = (Cerebrum.Data.SimpleView)dspwrk;
					for (int i = 0; i < tableDef.Tuples.Length; i++)
					{
						Cerebrum.ObjectHandle h = (Cerebrum.ObjectHandle)tableDef.GetValue(i, xObjectHandle);
						bool newChecked = (bool)tableDef.GetValue(i, xChecked);
						bool oldChecked = list.ContainsObject(h);
						if (oldChecked != newChecked)
						{
							if (newChecked)
							{
								using (Cerebrum.IComposite workgroup = this.DomainContext.AttachConnector(h))
								{
									list.Add(h, workgroup);
								}
							}
							else
							{
								list.Remove(list.CreateItemViewByObjectHandle(h));
							}
						}
					}
				}
			}
		}

		public void ReIndexDatabase()
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;

			Cerebrum.ObjectHandle hindxId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Components", "Name", "Cerebrum.Ganglion.Indexing.Controller");
			Cerebrum.ObjectHandle hringId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Attributes", "Name", "DependentInstances");

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "WebMessages"))
			{
				System.ComponentModel.PropertyDescriptor pdRep = view.GetItemProperties(null)["UniqueIdentity"];

				foreach (Cerebrum.Data.ComponentItemView item in view)
				{
					ReIndexReplicationInternal(domainContext, hindxId, hringId, pdRep, item);
				}
			}

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "Principals"))
			{
				System.ComponentModel.PropertyDescriptor pdRep = view.GetItemProperties(null)["UniqueIdentity"];

				foreach (Cerebrum.Data.ComponentItemView item in view)
				{
					ReIndexReplicationInternal(domainContext, hindxId, hringId, pdRep, item);
				}
			}

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "Workgroups"))
			{
				System.ComponentModel.PropertyDescriptor pdRep = view.GetItemProperties(null)["UniqueIdentity"];

				foreach (Cerebrum.Data.ComponentItemView item in view)
				{
					ReIndexReplicationInternal(domainContext, hindxId, hringId, pdRep, item);
				}
			}

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "Privileges"))
			{
				System.ComponentModel.PropertyDescriptor pdRep = view.GetItemProperties(null)["UniqueIdentity"];

				foreach (Cerebrum.Data.ComponentItemView item in view)
				{
					ReIndexReplicationInternal(domainContext, hindxId, hringId, pdRep, item);
				}
			}
		}
		public void ReIndexMessage(Cerebrum.ObjectHandle handle)
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;
			Cerebrum.ObjectHandle hindxId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Components", "Name", "Cerebrum.Ganglion.Indexing.Controller");
			Cerebrum.ObjectHandle hringId = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "Attributes", "Name", "DependentInstances");

			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "WebMessages"))
			{
				Cerebrum.Data.ComponentItemView item = view.CreateItemViewByObjectHandle(handle);
				System.ComponentModel.PropertyDescriptor pdUID = item.GetProperties()["UniqueIdentity"];

				ReIndexReplicationInternal(domainContext, hindxId, hringId, pdUID, item);
			}
		}
		private void ReIndexReplicationInternal(Cerebrum.Integrator.DomainContext domainContext, Cerebrum.ObjectHandle hIndexer, Cerebrum.ObjectHandle hringId, System.ComponentModel.PropertyDescriptor pdUID, Cerebrum.Data.ComponentItemView item)
		{
			string uniqueIdentity = Convert.ToString(pdUID.GetValue(item));
			if (uniqueIdentity == null || uniqueIdentity.Length < 1)
			{
				uniqueIdentity = Cerebrum.Ganglion.Remoting.Tools.NewReplicationId(false, false);
				pdUID.SetValue(item, uniqueIdentity);
			}
			else if(uniqueIdentity.Length==32)
			{
				uniqueIdentity = Cerebrum.Ganglion.Remoting.Tools.Encode32(130, Cerebrum.Ganglion.Remoting.Tools.Decode16(uniqueIdentity + "00"));
				pdUID.SetValue(item, uniqueIdentity);
			}
			Cerebrum.Ganglion.Ensemble.Tools.InvokeRecord(domainContext, "Components", hIndexer, "CreateIndex", new object[] { hringId, uniqueIdentity, item.ObjectHandle });
		}
		/*
		/// <summary>
		/// Add all WebMessages to DependentMessages of 00000000002gs8nabsur1p7cs4
		/// </summary>
		public void ReIndexDatabase2()
		{
			Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;

			Cerebrum.ObjectHandle hparent = Cerebrum.Management.Utilites.ResolveHandle(domainContext, "WebMessages", "UniqueIdentity", "00000000002gs8nabsur1p7cs4");
			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domainContext, "WebMessages"))
			{
				Cerebrum.Data.ComponentItemView parent = view.CreateItemViewByObjectHandle(hparent);
				using (Cerebrum.Data.SimpleView view2 = (Cerebrum.Data.SimpleView)parent["DependentMessages"])
				{
					foreach (Cerebrum.Data.ComponentItemView item in view)
					{
						if (hparent != item.ObjectHandle)
						{
							using (Cerebrum.IComposite composite = item.GetComposite())
							{
								view2.Add(item.ObjectHandle, composite);
							}
							using (Cerebrum.Data.SimpleView view3 = (Cerebrum.Data.SimpleView)item["PrecedentMessages"])
							{
								using (Cerebrum.IComposite cp = parent.GetComposite())
								{
									view3.Add(parent.ObjectHandle, cp);
								}
							}
						}
					}
				}
			}
		}
		*/
	}
}
