// Copyright (C) Dmitry Shuklin 2003-2007. All rights reserved.

using System;

namespace Cerebrum.Management
{
    /// <summary>
    /// Summary description for Tools.
    /// </summary>
    public sealed class Utilites
    {
        private Utilites()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        public static void InitTypeDescriptor(Cerebrum.Runtime.NativeSector sector, Cerebrum.ObjectHandle nativeHandleTypeId, string QualifiedTypeName, Cerebrum.ObjectHandle KernelObjectKindId)
        {
            using (Cerebrum.IConnector connector = Cerebrum.Reflection.TypeDescriptor.Create(sector, nativeHandleTypeId))
            {
                Cerebrum.Reflection.TypeDescriptor tds = (Cerebrum.Reflection.TypeDescriptor)connector.Component;
                tds.QualifiedTypeName = QualifiedTypeName;
                tds.KernelObjectKindId = KernelObjectKindId;
            }
        }

        public static void InitAttributeDescriptor(Cerebrum.Runtime.NativeSector sector, Cerebrum.ObjectHandle nativeHandleAttributeId, string Name, string DisplayName, Cerebrum.ObjectHandle nativeHandleTypeId)
        {
            using (Cerebrum.IConnector connector = Cerebrum.Reflection.AttributeDescriptor.CreateInstance(sector, nativeHandleAttributeId))
            {
                Cerebrum.Reflection.AttributeDescriptor ads = (Cerebrum.Reflection.AttributeDescriptor)connector.Component;
                if (Name != null)
                {
                    ads.Name = Name;
                }
                if (DisplayName != null)
                {
                    ads.DisplayName = DisplayName;
                }
                ads.AttributeTypeIdHandle = nativeHandleTypeId;
            }
        }

        public static void PrepAttributeDescriptor(Cerebrum.Runtime.NativeSector sector, Cerebrum.ObjectHandle nativeHandleAttributeId, string Name, string DisplayName)
        {
            using (Cerebrum.IConnector connector = sector.AttachConnector(nativeHandleAttributeId))
            {
                Cerebrum.Reflection.AttributeDescriptor ads = (Cerebrum.Reflection.AttributeDescriptor)connector.Component;
                ads.Name = Name;
                ads.DisplayName = DisplayName;
            }
        }

        /*public static AttributeDescriptor Create(Cerebrum.Runtime.NativeSector sector, Cerebrum.ObjectHandle attrHandle)
        {
            using(Cerebrum.IConnector connector = sector.CreateConnector(attrHandle, Cerebrum.Specialized.Concepts.AttributeDescriptorTypeId))
            {
                return (AttributeDescriptor)connector.Component;
            }
        }*/

        public static void InitTableDescriptor(Cerebrum.Runtime.NativeSector sector, Cerebrum.ObjectHandle nativeHandleAttributeId, string Name, string DisplayName, Cerebrum.ObjectHandle nativeHandleComponentTypeId, Cerebrum.ObjectHandle[] attributes)
        {
            using (Cerebrum.IConnector connector = Cerebrum.Reflection.TableDescriptor.CreateTable(sector, nativeHandleAttributeId, attributes))
            {
                Cerebrum.Reflection.TableDescriptor tbl = (Cerebrum.Reflection.TableDescriptor)connector.Component;
                tbl.Name = Name;
                tbl.DisplayName = DisplayName;
                tbl.ComponentTypeIdHandle = nativeHandleComponentTypeId;
            }
        }

        public static void InitDataBase(Cerebrum.IWorkspace domain)
        {

            using (Cerebrum.Runtime.NativeSector sector = domain.GetSector() as Cerebrum.Runtime.NativeSector)
            {

                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.ObjectHandleTypeId, "Cerebrum.ObjectHandle, Cerebrum.Typedef", Cerebrum.Runtime.KernelObjectKindId.Scalar);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.SystemStringTypeId, "System.String", Cerebrum.Runtime.KernelObjectKindId.Scalar);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.SystemInt16TypeId, "System.Int16", Cerebrum.Runtime.KernelObjectKindId.Scalar);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.SystemInt32TypeId, "System.Int32", Cerebrum.Runtime.KernelObjectKindId.Scalar);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.SystemInt64TypeId, "System.Int64", Cerebrum.Runtime.KernelObjectKindId.Scalar);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.SystemObjectTypeId, "System.Object", Cerebrum.Runtime.KernelObjectKindId.Scalar);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.SystemBooleanTypeId, "System.Boolean", Cerebrum.Runtime.KernelObjectKindId.Scalar);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.SystemDateTimeTypeId, "System.DateTime", Cerebrum.Runtime.KernelObjectKindId.Scalar);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.TypeDescriptorTypeId, "Cerebrum.Reflection.TypeDescriptor, Cerebrum.Integrator", Cerebrum.Runtime.KernelObjectKindId.Warden);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.AttributeDescriptorTypeId, "Cerebrum.Reflection.AttributeDescriptor, Cerebrum.Integrator", Cerebrum.Runtime.KernelObjectKindId.Warden);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.TableDescriptorTypeId, "Cerebrum.Reflection.TableDescriptor, Cerebrum.Integrator", Cerebrum.Runtime.KernelObjectKindId.Warden);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.MessageDescriptorTypeId, "Cerebrum.Reflection.MessageDescriptor, Cerebrum.Integrator", Cerebrum.Runtime.KernelObjectKindId.Warden);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.BindingListTypeId, "System.ComponentModel.IBindingList, System", Cerebrum.Runtime.KernelObjectKindId.Warden);
                //InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.KernelObjectClassTypeId, "Cerebrum.Runtime.KernelObjectKindId, Cerebrum.Runtime", Cerebrum.Runtime.KernelObjectKindId.Scalar);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.FieldDescriptorTypeId, "Cerebrum.Reflection.FieldDescriptor, Cerebrum.Integrator", Cerebrum.Runtime.KernelObjectKindId.Warden);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.GenericComponentTypeId, "Cerebrum.Integrator.GenericComponent, Cerebrum.Integrator", Cerebrum.Runtime.KernelObjectKindId.Warden);
                InitTypeDescriptor(sector, Cerebrum.Specialized.Concepts.DomainContextTypeId, "Cerebrum.Integrator.DomainContext, Cerebrum.Integrator", Cerebrum.ObjectHandle.Null);

                /*Cerebrum.Reflection.AttributeDescriptor atrObjectHandle;
                Cerebrum.Reflection.AttributeDescriptor atrName;
                Cerebrum.Reflection.AttributeDescriptor atrDisplayName;
                Cerebrum.Reflection.AttributeDescriptor atrDescription;
                Cerebrum.Reflection.AttributeDescriptor atrTypeId;
                Cerebrum.Reflection.AttributeDescriptor atrDefTypeId;
                Cerebrum.Reflection.AttributeDescriptor atrQualifiedTypeName;
                Cerebrum.Reflection.AttributeDescriptor atrKernelObjectClass;
                Cerebrum.Reflection.AttributeDescriptor atrSelectedAttributes;
                Cerebrum.Reflection.AttributeDescriptor atrSelectedComponents;
                Cerebrum.Reflection.AttributeDescriptor atrPrecedentTableId;
                Cerebrum.Reflection.AttributeDescriptor atrDependentFieldId;
                Cerebrum.Reflection.AttributeDescriptor atrDisplayFieldId;
*/
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.PlasmaHandleAttribute, null, null, Cerebrum.Specialized.Concepts.ObjectHandleTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.NameAttribute, null, null, Cerebrum.Specialized.Concepts.SystemStringTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.DisplayNameAttribute, null, null, Cerebrum.Specialized.Concepts.SystemStringTypeId);

                PrepAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.PlasmaHandleAttribute, Cerebrum.Specialized.KnownNames.ObjectHandle, "Object Handle");
                PrepAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.NameAttribute, "Name", "Name");
                PrepAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.DisplayNameAttribute, "DisplayName", "DisplayName");

                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.DescriptionAttribute, "Description", "Description", Cerebrum.Specialized.Concepts.SystemStringTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.GroundHandleAttribute, "GroundHandle", "Ground handle", Cerebrum.Specialized.Concepts.ObjectHandleTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.KindIdHandleAttribute, "KindIdHandle", "KindId handle", Cerebrum.Specialized.Concepts.ObjectHandleTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.TypeIdHandleAttribute, "TypeIdHandle", "TypeId handle", Cerebrum.Specialized.Concepts.ObjectHandleTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.DefaultTypeIdAttribute, "DefaultTypeId", "Default TypeId handle", Cerebrum.Specialized.Concepts.ObjectHandleTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.QualifiedTypeNameAttribute, "QualifiedTypeName", "Qualified Type Name", Cerebrum.Specialized.Concepts.SystemStringTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.KernelObjectKindIdAttribute, "KernelObjectClass", "Kernel Object KindId", Cerebrum.Specialized.Concepts.ObjectHandleTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.SelectedAttributesAttribute, "SelectedAttributes", "Selected Attributes", Cerebrum.Specialized.Concepts.BindingListTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.SelectedComponentsAttribute, "SelectedComponents", "Selected Components", Cerebrum.Specialized.Concepts.BindingListTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.PrecedentTableIdAttribute, "PrecedentTableId", "Precedent Table Id", Cerebrum.Specialized.Concepts.ObjectHandleTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.DependentFieldIdAttribute, "DependentFieldId", "Dependent Field Id", Cerebrum.Specialized.Concepts.ObjectHandleTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.DisplayFieldIdAttribute, "DisplayFieldId", "Display Field Id", Cerebrum.Specialized.Concepts.ObjectHandleTypeId);
                InitAttributeDescriptor(sector, Cerebrum.Specialized.Concepts.DerivedFlagAttribute, "Derived", "Derived", Cerebrum.Specialized.Concepts.SystemBooleanTypeId);

                Cerebrum.IConnector cntAttributes = Cerebrum.Reflection.TableDescriptor.CreateTable(sector, Cerebrum.Specialized.Concepts.AttributesTableId,
                    new Cerebrum.ObjectHandle[] {
															 Cerebrum.Specialized.Concepts.PlasmaHandleAttribute, 
															 Cerebrum.Specialized.Concepts.TypeIdHandleAttribute, 
															 Cerebrum.Specialized.Concepts.NameAttribute, 
															 Cerebrum.Specialized.Concepts.DisplayNameAttribute, 
															 Cerebrum.Specialized.Concepts.DescriptionAttribute, 
															 Cerebrum.Specialized.Concepts.DefaultTypeIdAttribute});
                Cerebrum.Reflection.TableDescriptor tblAttributes = cntAttributes.Component as Cerebrum.Reflection.TableDescriptor;
                tblAttributes.Name = "Attributes";
                tblAttributes.DisplayName = "Attributes";
                tblAttributes.ComponentTypeIdHandle = Cerebrum.Specialized.Concepts.AttributeDescriptorTypeId;

                Cerebrum.Data.TableView vwv;
                vwv = tblAttributes.GetTableView();
                vwv.Add(sector, Cerebrum.Specialized.Concepts.PlasmaHandleAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.TypeIdHandleAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.NameAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.DisplayNameAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.DescriptionAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.DefaultTypeIdAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.QualifiedTypeNameAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.KernelObjectKindIdAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.SelectedAttributesAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.SelectedComponentsAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.PrecedentTableIdAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.DependentFieldIdAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.DisplayFieldIdAttribute);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.DerivedFlagAttribute);
                vwv.Dispose();

                Cerebrum.IConnector cntTypes = Cerebrum.Reflection.TableDescriptor.CreateTable(sector, Cerebrum.Specialized.Concepts.TypesTableId,
                    new Cerebrum.ObjectHandle[] {
															Cerebrum.Specialized.Concepts.PlasmaHandleAttribute, 
															Cerebrum.Specialized.Concepts.TypeIdHandleAttribute, 
															Cerebrum.Specialized.Concepts.NameAttribute, 
															Cerebrum.Specialized.Concepts.DisplayNameAttribute, 
															Cerebrum.Specialized.Concepts.DescriptionAttribute, 
															Cerebrum.Specialized.Concepts.QualifiedTypeNameAttribute, 
															Cerebrum.Specialized.Concepts.KernelObjectKindIdAttribute});
                Cerebrum.Reflection.TableDescriptor tblTypes = cntTypes.Component as Cerebrum.Reflection.TableDescriptor;
                tblTypes.Name = "Types";
                tblTypes.DisplayName = "Types";
                tblTypes.ComponentTypeIdHandle = Cerebrum.Specialized.Concepts.TypeDescriptorTypeId;

                vwv = tblTypes.GetTableView();
                vwv.Add(sector, Cerebrum.Specialized.Concepts.ObjectHandleTypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.SystemObjectTypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.SystemStringTypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.SystemInt16TypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.SystemInt32TypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.SystemInt64TypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.SystemBooleanTypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.SystemDateTimeTypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.TypeDescriptorTypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.AttributeDescriptorTypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.TableDescriptorTypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.MessageDescriptorTypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.BindingListTypeId);
                //vwv.Add(sector, Cerebrum.Specialized.Concepts.KernelObjectClassTypeId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.FieldDescriptorTypeId);
				vwv.Add(sector, Cerebrum.Specialized.Concepts.GenericComponentTypeId);
				vwv.Add(sector, Cerebrum.Specialized.Concepts.DomainContextTypeId);
				vwv.Dispose();

                InitTableDescriptor(sector, Cerebrum.Specialized.Concepts.MessagesTableId, "Messages", "Messages", Cerebrum.Specialized.Concepts.MessageDescriptorTypeId,
                    new Cerebrum.ObjectHandle[] {
															Cerebrum.Specialized.Concepts.PlasmaHandleAttribute, 
															Cerebrum.Specialized.Concepts.TypeIdHandleAttribute, 
															Cerebrum.Specialized.Concepts.NameAttribute, 
															Cerebrum.Specialized.Concepts.DisplayNameAttribute, 
															Cerebrum.Specialized.Concepts.DescriptionAttribute});

                InitTableDescriptor(sector, Cerebrum.Specialized.Concepts.FieldsTableId, "Fields", "Fields", Cerebrum.Specialized.Concepts.FieldDescriptorTypeId,
                    new Cerebrum.ObjectHandle[] {
															Cerebrum.Specialized.Concepts.PlasmaHandleAttribute, 
															Cerebrum.Specialized.Concepts.TypeIdHandleAttribute, 
															Cerebrum.Specialized.Concepts.DependentFieldIdAttribute, 
															Cerebrum.Specialized.Concepts.PrecedentTableIdAttribute, 
															Cerebrum.Specialized.Concepts.DerivedFlagAttribute});

                InitTableDescriptor(sector, Cerebrum.Specialized.Concepts.TablesTableId, "Tables", "Tables", Cerebrum.Specialized.Concepts.TableDescriptorTypeId,
                    new Cerebrum.ObjectHandle[] {
															Cerebrum.Specialized.Concepts.PlasmaHandleAttribute, 
															Cerebrum.Specialized.Concepts.TypeIdHandleAttribute, 
															Cerebrum.Specialized.Concepts.NameAttribute, 
															Cerebrum.Specialized.Concepts.DisplayNameAttribute, 
															Cerebrum.Specialized.Concepts.DescriptionAttribute, 
															Cerebrum.Specialized.Concepts.DefaultTypeIdAttribute, 
															Cerebrum.Specialized.Concepts.DisplayFieldIdAttribute, 
															Cerebrum.Specialized.Concepts.SelectedAttributesAttribute, 
															Cerebrum.Specialized.Concepts.SelectedComponentsAttribute});

                Cerebrum.IConnector cntTables = sector.AttachConnector(Cerebrum.Specialized.Concepts.TablesTableId);
                Cerebrum.Reflection.TableDescriptor tblTables = cntTables.Component as Cerebrum.Reflection.TableDescriptor;

                vwv = tblTables.GetTableView();
                vwv.Add(sector, Cerebrum.Specialized.Concepts.TypesTableId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.TablesTableId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.FieldsTableId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.AttributesTableId);
                vwv.Add(sector, Cerebrum.Specialized.Concepts.MessagesTableId);
                vwv.Dispose();

                // inherited table support
                sector.Newobj(Cerebrum.Specialized.Concepts.SingleObjectHandleVectorId, Cerebrum.Runtime.KernelObjectKindId.Warden);
                using (Cerebrum.IContainer c1 = sector.AttachConnector(Cerebrum.Specialized.Concepts.SingleObjectHandleVectorId) as Cerebrum.IContainer)
                {
                    //vector.SetMap(Cerebrum.Specialized.Concepts.PlasmaHandleAttribute, Cerebrum.Runtime.MapAccess.Scalar, new Cerebrum.ObjectHandle(1));
                    using (Cerebrum.IComposite c2 = sector.AttachConnector(Cerebrum.Specialized.Concepts.PlasmaHandleAttribute))
                    {
                        c1.EngageConnector(Cerebrum.Specialized.Concepts.PlasmaHandleAttribute, c2);
                    }
                }

                cntTypes.Dispose();
                cntTables.Dispose();
                cntAttributes.Dispose();
            }
			domain.Milestones.Flush();
        }

        public static void ExportDatabase(Cerebrum.IWorkspace domain, System.Xml.XmlWriter xw)
        {
            using (Cerebrum.Runtime.NativeSector sector = domain.GetSector() as Cerebrum.Runtime.NativeSector)
            {
                xw.WriteStartDocument(false);
                xw.WriteDocType("vnpidb", null, null/*"vnpiodl.dtd"*/, null);
                xw.WriteStartElement("SystemDatabase");

                xw.WriteStartElement("Globals");
                xw.WriteAttributeString("SectorCurrentSequence", sector.CurrSequence.ToInt64().ToString());
                xw.WriteEndElement();

                using (Cerebrum.IConnector connector = sector.AttachConnector(Cerebrum.Specialized.Concepts.TablesTableId))
                {
                    Cerebrum.Reflection.TableDescriptor tblTables = connector.Component as Cerebrum.Reflection.TableDescriptor;
                    Cerebrum.Data.TableView vwv = tblTables.GetTableView();
                    Cerebrum.Data.TableSet tst = new Cerebrum.Data.TableSet(vwv);
                    System.ComponentModel.PropertyDescriptorCollection prpsTables = tst.GetItemProperties(null);

                    //Cerebrum.Integrator.Xml.SystemDatabase sysdb = new Cerebrum.Integrator.Xml.SystemDatabase();
                    //sysdb.EnforceConstraints = false;

                    foreach (System.ComponentModel.PropertyDescriptor prpTbl in prpsTables)
                    {
                        Cerebrum.ObjectHandle defaultTypeHandle = Cerebrum.ObjectHandle.Null;
                        if (prpTbl is Cerebrum.Data.CerebrumTableDescriptor)
                        {
                            defaultTypeHandle = (prpTbl as Cerebrum.Data.CerebrumTableDescriptor).TableView.TableDescriptor.ComponentTypeIdHandle;
                        }
                        System.ComponentModel.IBindingList list = (System.ComponentModel.IBindingList)prpTbl.GetValue(tst[0]);
                        System.ComponentModel.PropertyDescriptorCollection itemPrps = (list as System.ComponentModel.ITypedList).GetItemProperties(null);
                        foreach (object item in list)
                        {
                            xw.WriteStartElement(prpTbl.Name);
                            foreach (System.ComponentModel.PropertyDescriptor itemPrp in itemPrps)
                            {
                                if (itemPrp is Cerebrum.Data.AttributePropertyDescriptor)
                                {
                                    if ((itemPrp as Cerebrum.Data.AttributePropertyDescriptor).Derived)
                                    {
                                        continue;
                                    }
                                }
                                if (itemPrp.PropertyType == typeof(Cerebrum.ObjectHandle))
                                {
                                    Cerebrum.ObjectHandle h = Cerebrum.ObjectHandle.Null;
                                    object o = itemPrp.GetValue(item);
                                    if (o != null && o != System.DBNull.Value)
                                    {
                                        h = (Cerebrum.ObjectHandle)o;
                                        if (itemPrp.Name == Cerebrum.Specialized.KnownNames.TypeIdHandle)
                                        {
                                            if (h == defaultTypeHandle) continue;
                                        }

                                        xw.WriteAttributeString(itemPrp.Name, h.ToString());
                                    }
                                }
                                else if (itemPrp.PropertyType == typeof(System.ComponentModel.IBindingList))
                                {
                                    continue;
                                }
                                else if (itemPrp.PropertyType == typeof(System.Object))
                                {
                                    continue;
                                }
                                else
                                {
                                    object o = itemPrp.GetValue(item);
                                    if (o != null && o != System.DBNull.Value)
                                    {
                                        xw.WriteAttributeString(itemPrp.Name, Convert.ToString(o));
                                    }
                                }
                            }
                            foreach (System.ComponentModel.PropertyDescriptor itemPrp in itemPrps)
                            {
                                if (itemPrp.PropertyType == typeof(System.ComponentModel.IBindingList))
                                {
                                    if (prpTbl.Name == "Tables" && itemPrp.Name == "SelectedComponents") continue;
                                    System.ComponentModel.IBindingList attrList = (System.ComponentModel.IBindingList)itemPrp.GetValue(item);
                                    System.ComponentModel.PropertyDescriptor attrNH = (attrList as System.ComponentModel.ITypedList).GetItemProperties(null)[Cerebrum.Specialized.KnownNames.ObjectHandle];
                                    foreach (object itemNH in attrList)
                                    {
                                        xw.WriteStartElement(itemPrp.Name);
                                        Cerebrum.ObjectHandle h;
                                        /*if(itemNH is Cerebrum.Data.ComponentItemView && itemPrp.Name=="SelectedAttributes")
                                        {
                                            using(Cerebrum.IConnector iconnector = (itemNH as Cerebrum.Data.ComponentItemView).GetConnector())
                                            {
                                                h = (iconnector.Component as Cerebrum.Reflection.IAttributeDescriptor).AttributePlasmaHandle;
                                            }
                                        }
                                        else
                                        {
                                            h = (Cerebrum.ObjectHandle)attrNH.GetValue(itemNH);
                                        }*/
                                        h = (Cerebrum.ObjectHandle)attrNH.GetValue(itemNH);

                                        xw.WriteAttributeString(attrNH.Name, ((Cerebrum.ObjectHandle)h).ToString());
                                        xw.WriteEndElement();
                                    }
                                    if (attrList is System.IDisposable) ((System.IDisposable)attrList).Dispose();
                                }
                                else if (itemPrp.PropertyType == typeof(System.Object))
                                {
                                    object o = itemPrp.GetValue(item);
                                    if (o != null && o != System.DBNull.Value)
                                    {
                                        using (System.IO.MemoryStream ms = new System.IO.MemoryStream())
                                        {
                                            System.Runtime.Serialization.Formatters.Binary.BinaryFormatter formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
                                            formatter.Serialize(ms, o);
                                            xw.WriteStartElement(itemPrp.Name);
                                            xw.WriteString(Convert.ToBase64String(ms.ToArray()));
                                            xw.WriteEndElement();
                                        }
                                    }
                                }
                            }
                            xw.WriteEndElement();
                        }
                        if (list is System.IDisposable) ((System.IDisposable)list).Dispose();
                    }
                    vwv.Dispose();
                    tst.Dispose();

                }
                xw.WriteEndElement();
                xw.WriteEndDocument();
                xw.Flush();
            }
        }
        public static void ImportDatabase(Cerebrum.IWorkspace domain, System.Xml.XmlReader xr)
        {
            ImportDatabase(domain, xr, null);
        }
        public static void ImportDatabase(Cerebrum.IWorkspace domain, System.Xml.XmlReader xr, System.Collections.IDictionary handleMap)
        {
            System.Collections.ArrayList lateBinding = new System.Collections.ArrayList();
            System.Xml.XmlDocument xd = new System.Xml.XmlDocument();
            xd.Load(xr);
            System.Xml.XmlNode root = xd.SelectSingleNode("SystemDatabase");

            using (Cerebrum.Runtime.NativeSector sector = domain.GetSector() as Cerebrum.Runtime.NativeSector)
            {
                if (handleMap == null)
                {
					System.Xml.XmlNode nodeGlobals = root.SelectSingleNode("Globals");
					if(nodeGlobals!=null)
					{
						sector.CurrSequence = new Cerebrum.ObjectHandle(Convert.ToInt64(nodeGlobals.Attributes["SectorCurrentSequence"].Value));
					}
                }

                using (Cerebrum.IConnector connector = sector.AttachConnector(Cerebrum.Specialized.Concepts.TablesTableId))
                {

                    Cerebrum.Reflection.TableDescriptor tblTables = connector.Component as Cerebrum.Reflection.TableDescriptor;
                    Cerebrum.Data.TableView vwvTables = tblTables.GetTableView();
                    Cerebrum.Data.TableSet tst = new Cerebrum.Data.TableSet(vwvTables);
                    System.ComponentModel.PropertyDescriptorCollection prps = tst.GetItemProperties(null);

                    Cerebrum.Data.TableView vwvTypes = (Cerebrum.Data.TableView)prps["Types"].GetValue(tst[0]);
                    Cerebrum.Data.TableView vwvAttributes = (Cerebrum.Data.TableView)prps["Attributes"].GetValue(tst[0]);

                    ImportTable(sector, handleMap, "Types", vwvTypes, root, lateBinding);
                    ImportTable(sector, handleMap, "Attributes", vwvAttributes, root, lateBinding);

                    Cerebrum.Data.TableView vwvFields = (Cerebrum.Data.TableView)prps["Fields"].GetValue(tst[0]);

                    ImportTable(sector, handleMap, "Fields", vwvFields, root, lateBinding);
                    ImportTable(sector, handleMap, "Tables", vwvTables, root, lateBinding);

                    vwvFields.Dispose();
                    vwvTables.Dispose();

                    tblTables.ResetDefaultView();

                    vwvTables = tblTables.GetTableView();
                    tst.Dispose();
                    tst = new Cerebrum.Data.TableSet(vwvTables);
                    prps = tst.GetItemProperties(null);

                    //System.ComponentModel.PropertyDescriptor prpt = prps["UiServices"];

                    foreach (System.ComponentModel.PropertyDescriptor prp in prps)
                    {
                        if (prp.Name == "Types" || prp.Name == "Attributes" || prp.Name == "Tables") continue;

                        System.ComponentModel.IBindingList list = (System.ComponentModel.IBindingList)prp.GetValue(tst[0]);
                        if (list is Cerebrum.Data.TableView)
                        {
                            ImportTable(sector, handleMap, prp.Name, list as Cerebrum.Data.TableView, root, lateBinding);
                        }
                        //if(list is System.IDisposable) ((System.IDisposable)list).Dispose();
                    }
                    foreach (VectorLateBinding entry in lateBinding)
                    {
                        System.ComponentModel.PropertyDescriptor prpTbl = prps[entry.TableName];
                        Cerebrum.Data.TableView list = prpTbl.GetValue(tst[0]) as Cerebrum.Data.TableView;
                        if (list != null)
                        {
                            System.ComponentModel.PropertyDescriptorCollection itemPrps = list.GetItemProperties(null);
                            System.ComponentModel.PropertyDescriptor prpFld = itemPrps[entry.FieldName];
                            using (Cerebrum.Data.VectorView attrList = (Cerebrum.Data.VectorView)prpFld.GetValue(list.CreateItemViewByObjectHandle(entry.ComponentHandle)))
                            {
                                System.ComponentModel.PropertyDescriptor attrNH = attrList.GetItemProperties(null)[Cerebrum.Specialized.KnownNames.ObjectHandle];
                                object itemNH = attrList.AddNew();
                                attrNH.SetValue(itemNH, entry.AttributeHandle);
                            }
                        }
                    }
                    tst.Dispose();
                    vwvTables.Dispose();
                    vwvTypes.Dispose();
                    vwvAttributes.Dispose();
                }
            }
            //FixrelDatabase(domain);
			domain.Milestones.Flush();
        }
        public static void ImportTable(Cerebrum.Runtime.NativeSector sector, System.Collections.IDictionary handleMap, string name, Cerebrum.Data.TableView tblView, System.Xml.XmlNode root, System.Collections.ArrayList lateBinding)
        {
            System.Xml.XmlNodeList nodes = root.SelectNodes(name);
            System.ComponentModel.PropertyDescriptorCollection itemPrps = tblView.GetItemProperties(null);

            foreach (System.Xml.XmlNode node in nodes)
            {
                object item;
                Cerebrum.ObjectHandle h = ReMapHandle(sector, handleMap, node.Attributes[Cerebrum.Specialized.KnownNames.ObjectHandle].Value);
                if (tblView.ContainsObject(h))
                {
                    item = tblView.CreateItemViewByObjectHandle(h);
                }
                else
                {
                    Cerebrum.IComposite composite = sector.AttachConnector(h);
                    if (composite == null)
                    {
                        Cerebrum.ObjectHandle TypeIdHandle = Cerebrum.ObjectHandle.Null;

                        string id = null;
                        System.Xml.XmlNode n = node.Attributes[Cerebrum.Specialized.KnownNames.TypeIdHandle];
                        if (n != null)
                        {
                            id = n.Value;
                        }
                        if (id != null && id.Length > 0)
                        {
                            TypeIdHandle = ReMapHandle(sector, handleMap, id);
                        }

                        if (TypeIdHandle == Cerebrum.ObjectHandle.Null)
                        {
                            item = tblView.AddNew(h);
                        }
                        else
                        {
                            item = tblView.AddNew(h, TypeIdHandle);
                        }
                    }
                    else
                    {
                        item = tblView[tblView.Add(h, composite)];
                    }
                }

                foreach (System.ComponentModel.PropertyDescriptor itemPrp in itemPrps)
                {
                    if (itemPrp.PropertyType == typeof(System.ComponentModel.IBindingList))
                    {
                        if (name == "Tables" && itemPrp.Name == "SelectedComponents") continue;
                        System.Xml.XmlNodeList listItems = node.SelectNodes(itemPrp.Name);
                        if (listItems != null)
                        {
                            //import table attributes
                            using (Cerebrum.Data.VectorView attrList = (Cerebrum.Data.VectorView)itemPrp.GetValue(item))
                            {
                                System.ComponentModel.PropertyDescriptor attrNH = attrList.GetItemProperties(null)[Cerebrum.Specialized.KnownNames.ObjectHandle];
                                foreach (System.Xml.XmlNode n in listItems)
                                {
                                    Cerebrum.ObjectHandle hNH = ReMapHandle(sector, handleMap, n.Attributes[Cerebrum.Specialized.KnownNames.ObjectHandle].Value);
                                    Cerebrum.ObjectHandle h0 = hNH;
                                    bool newAttr = !attrList.ContainsObject(h0);
                                    if (newAttr && itemPrp.Name == "SelectedAttributes")
                                    {
                                        using (Cerebrum.IComposite composite = sector.AttachConnector(hNH))
                                        {
                                            h0 = (composite.Component as Cerebrum.Reflection.IAttributeDescriptor).AttributePlasmaHandle(composite);
                                        }
                                        newAttr = !attrList.ContainsObject(h0);
                                    }
                                    if (newAttr)
                                    {
                                        using (Cerebrum.IConnector iconnector = sector.AttachConnector(hNH))
                                        {
                                            if (iconnector != null)
                                            {
                                                object itemNH = attrList.AddNew();
                                                attrNH.SetValue(itemNH, hNH);
                                            }
                                            else
                                            {
                                                VectorLateBinding entry = new VectorLateBinding();
                                                entry.TableName = name;
                                                entry.FieldName = itemPrp.Name;
                                                entry.ComponentHandle = (item as Cerebrum.Data.ComponentItemView).ObjectHandle;
                                                entry.AttributeHandle = hNH;
                                                lateBinding.Add(entry);
                                            }
                                        }
                                    }
                                    else if (h0 != hNH)
                                    {
                                        using (Cerebrum.IConnector c2 = sector.AttachConnector(hNH))
                                        {
                                            attrList.CreateItemViewByObjectHandle(h0).SetConnector(hNH, c2);
                                        }
                                    }
                                }
                            }
                        }
                    }
                    else if (itemPrp.PropertyType == typeof(object))
                    {
                        System.Xml.XmlNode itemNode = node.SelectSingleNode(itemPrp.Name);
                        if (itemNode != null)
                        {
                            string val = itemNode.InnerText;
                            if (!(val == null || Convert.IsDBNull(val)))
                            {
                                using (System.IO.MemoryStream ms = new System.IO.MemoryStream(Convert.FromBase64String(val)))
                                {
                                    System.Runtime.Serialization.Formatters.Binary.BinaryFormatter formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
                                    object o = formatter.Deserialize(ms);
                                    if (!(o == null || Convert.IsDBNull(o)))
                                    {
                                        itemPrp.SetValue(item, o);
                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        System.Xml.XmlNode itemNode = node.Attributes[itemPrp.Name];
                        if (itemNode != null)
                        {
                            string val = itemNode.Value;
                            if (itemPrp.PropertyType == typeof(Cerebrum.ObjectHandle))
                            {
                                if (val == null || val.Length < 1)
                                {
                                    itemPrp.SetValue(item, Cerebrum.ObjectHandle.Null);
                                }
                                else
                                {
                                    itemPrp.SetValue(item, ReMapHandle(sector, handleMap, val));
                                }
                            }
                            else
                            {
                                if (!(val == null || Convert.IsDBNull(val)))
                                {
                                    itemPrp.SetValue(item, val);
                                }
                            }
                        }
                    }
                }
            }
        }
        public static Cerebrum.ObjectHandle ReMapHandle(Cerebrum.Runtime.NativeSector sector, System.Collections.IDictionary handleMap, string shandle)
        {
            Cerebrum.ObjectHandle handle;
            if (shandle[0] == '#')
            {
                handle = Cerebrum.ObjectHandle.Parse(shandle.Substring(1));
                return handle;
            }
            else if (shandle[0] == '?')
            {
                string path = shandle.Substring(1);
                int dot = path.IndexOf('.');
                int eqv = path.IndexOf('=', dot + 1);
                int q01 = path.IndexOf('\'', eqv + 1);
                int q02 = path.IndexOf('\'', q01 + 1);
                string table = path.Substring(0, dot);
                string field = path.Substring(dot + 1, eqv - dot - 1);
                string value = path.Substring(q01 + 1, q02 - q01 - 1);

                using (Cerebrum.IConnector connector = Cerebrum.Integrator.DomainContext.FromConnector(sector).GetTable(table))
                {
                    using (Cerebrum.Data.TableView view = (connector.Component as Cerebrum.Reflection.TableDescriptor).GetTableView())
                    {
                        System.ComponentModel.PropertyDescriptor descriptor = view.GetItemProperties(null)[field];
                        handle = FindHandle(view, descriptor, value);
                        return handle;
                    }
                }

            }
            else
            {
                handle = Cerebrum.ObjectHandle.Parse(shandle);
            }
            if (handleMap == null) return handle;
            if (((System.IComparable)handle).CompareTo(Cerebrum.Specialized.Concepts.FirstUserHandle) < 0)
            {
                return handle;
            }
            else
            {
                if (handleMap.Contains(handle))
                {
                    return (Cerebrum.ObjectHandle)handleMap[handle];
                }
                else
                {
                    Cerebrum.ObjectHandle newHandle = sector.NextSequence();
                    handleMap.Add(handle, newHandle);
                    return newHandle;
                }
            }
        }

        public static Cerebrum.Data.TableView GetView(Cerebrum.Integrator.DomainContext domainContext, string tableName)
        {
            using (Cerebrum.IConnector connector = domainContext.GetTable(tableName))
            {
				if(connector==null)
				{
					return null;
				}
				else
				{
					Cerebrum.Reflection.TableDescriptor tbd = connector.Component as Cerebrum.Reflection.TableDescriptor;
					return tbd == null ? null : tbd.GetTableView();
				}
            }
            /*using(Cerebrum.Data.TableView vwvTables = domainContext.TablesView)
            {
                using(Cerebrum.Data.TableSet tst = new Cerebrum.Data.TableSet(vwvTables))
                {
                    System.ComponentModel.PropertyDescriptorCollection prps = tst.GetItemProperties(null);
                    System.ComponentModel.PropertyDescriptor prp = prps[tableName];
			
                    return prp==null?null:((Cerebrum.Data.TableView)prp.GetValue(tst[0]));
                }
            }*/
        }
        public static void Fixrel2Database(Cerebrum.Runtime.INativeDomain domain)
        {
            using (Cerebrum.Runtime.NativeSector sector = domain.GetSector() as Cerebrum.Runtime.NativeSector)
            {
                using (Cerebrum.IConnector conTables = sector.AttachConnector(Cerebrum.Specialized.Concepts.TablesTableId))
                {

                    Cerebrum.Reflection.TableDescriptor tblTables = conTables.Component as Cerebrum.Reflection.TableDescriptor;
                    Cerebrum.Data.TableView vwvTables = tblTables.GetTableView();
                    Cerebrum.Data.TableSet tst = new Cerebrum.Data.TableSet(vwvTables);
                    System.ComponentModel.PropertyDescriptorCollection prpsTables = tst.GetItemProperties(null);

                    System.Collections.Hashtable hash = new System.Collections.Hashtable();
                    System.ComponentModel.PropertyDescriptor prpRelation = prpsTables["Relations"];

                    if (prpRelation != null)
                    {
                        Cerebrum.Data.TableView vwvRelations = (Cerebrum.Data.TableView)prpRelation.GetValue(tst[0]);

                        for (int i = 0; i < vwvRelations.Count; i++)
                        {
                            using (Cerebrum.IComposite conRelation = vwvRelations[i].GetComposite())
                            {
                                Cerebrum.Reflection.FieldDescriptor field = conRelation.Component as Cerebrum.Reflection.FieldDescriptor;
                                hash.Add(field.AttributePlasmaHandle(conRelation), conRelation.PlasmaHandle);
                            }
                        }

                        vwvRelations.Dispose();

                        for (int i = 0; i < vwvTables.Count; i++)
                        {
                            using (Cerebrum.IConnector conTable = vwvTables[i].GetComposite())
                            {
                                Cerebrum.Reflection.TableDescriptor table = conTable.Component as Cerebrum.Reflection.TableDescriptor;
                                using (Cerebrum.Runtime.NativeWarden columns = table.GetSelectedAttributesVector())
                                {
                                    for (int c = 0; c < columns.Count; c++)
                                    {
                                        Cerebrum.ObjectHandle h = columns.GetMap(new Cerebrum.ObjectHandle(c + 1));
                                        if (hash.ContainsKey(h))
                                        {
                                            columns.SetMap(new Cerebrum.ObjectHandle(c + 1), Cerebrum.Runtime.MapAccess.Scalar, (Cerebrum.ObjectHandle)hash[h]);
                                            throw new System.Exception("");
                                        }
                                    }
                                }
                            }
                        }
                    }
                    tst.Dispose();
                    vwvTables.Dispose();
                }
            }
        }


        private static System.Collections.Hashtable m_ResolverHashTable = null;

        /// <summary>
        ///  ObjectHandle   .
        /// </summary>
        /// <param name="name"> </param>
        /// <returns>ObjectHandle </returns>
		public static Cerebrum.ObjectHandle ResolveHandle(Cerebrum.Integrator.DomainContext context, string TableName, string FieldName, object value)
		{
			object key = Utilites.GetResolverKey(TableName, FieldName);
			System.Collections.Hashtable resolver = context.Properties[key] as System.Collections.Hashtable;

			if (resolver == null)
			{
				resolver = new System.Collections.Hashtable();
				RefreshResolver(resolver, context, TableName, FieldName);
				context.Properties[key] = resolver;
			}

			object h = resolver[value];
			if (h == null)
			{
				return Cerebrum.ObjectHandle.Null;
			}
			else
			{
				return (Cerebrum.ObjectHandle)h;
			}
		}
        private static object GetResolverKey(string TableName, string FieldName)
        {
            if (m_ResolverHashTable == null)
            {
                m_ResolverHashTable = new System.Collections.Hashtable();
            }

            string keyName = TableName + "." + FieldName;
            object key;
            if (m_ResolverHashTable.ContainsKey(keyName))
            {
                key = m_ResolverHashTable[keyName];
            }
            else
            {
                key = new object();
                m_ResolverHashTable[keyName] = key;
            }
			return key;
        }
		private static void RefreshResolver(System.Collections.Hashtable resolver, Cerebrum.Integrator.DomainContext context, string TableName, string FieldName)
		{
			//     Cerebrum.
			using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(context, TableName))
			{
				System.ComponentModel.PropertyDescriptor pd = view.GetItemProperties(null)[FieldName];
				//      .
				for (int i = 0; i < view.Count; i++)
				{
					object v = pd.GetValue(view[i]);
					resolver[v] = view[i].ObjectHandle;
				}
			}
		}

		public static void RefreshHandle(Cerebrum.Integrator.DomainContext context, string TableName, string FieldName, Cerebrum.ObjectHandle handle)
		{
			object key = Utilites.GetResolverKey(TableName, FieldName);
			System.Collections.Hashtable resolver = context.Properties[key] as System.Collections.Hashtable;

			if (resolver == null)
			{
				resolver = new System.Collections.Hashtable();
				RefreshResolver(resolver, context, TableName, FieldName);
				context.Properties[key] = resolver;
			}
			else
			{
				using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(context, TableName))
				{
					if(view.ContainsObject(handle))
					{
						System.ComponentModel.PropertyDescriptor pd = view.GetItemProperties(null)[FieldName];
						Cerebrum.Data.ComponentItemView item = view.CreateItemViewByObjectHandle(handle);
						object v = pd.GetValue(item);
						resolver[v] = handle;
					}
				}
			}
		}

		public static void RefreshHandles(Cerebrum.Integrator.DomainContext context, string TableName, string FieldName)
		{
			object key = Utilites.GetResolverKey(TableName, FieldName);
			System.Collections.Hashtable resolver = new System.Collections.Hashtable();
			RefreshResolver(resolver, context, TableName, FieldName);
			context.Properties[key] = resolver;
		}

        public static Cerebrum.ObjectHandle FindHandle(Cerebrum.Data.TableView view, System.ComponentModel.PropertyDescriptor prp, string value)
        {
            int i = view.Find(prp, value);
            if (i >= 0)
            {
                return view[i].ObjectHandle;
            }
            else
            {
                return Cerebrum.ObjectHandle.Null;
            }
        }

		private static Cerebrum.Security.CerebrumPrincipal LoadPrincipal(Cerebrum.ObjectHandle prh, Cerebrum.Management.Security.Reflection.PrincipalDescriptor prd, bool isAuthenticated)
		{
			Cerebrum.Security.CerebrumIdentity cid = new Cerebrum.Security.CerebrumIdentity(prh, prd.Name, prd.DisplayName, prd.Description, "Cerebrum.Security", isAuthenticated);
			System.Collections.Hashtable hshPriveleges = new System.Collections.Hashtable();
			System.Collections.ArrayList arrPriveleges = new System.Collections.ArrayList();
			System.Collections.ArrayList arrWorkgroups = new System.Collections.ArrayList();
			using(Cerebrum.IContainer workgroups = prd.GetSelectedWorkgroupsVector())
			{
				if(workgroups!=null)
				{
					foreach(System.Collections.DictionaryEntry de in (System.Collections.IEnumerable)workgroups)
					{
						Cerebrum.ObjectHandle wh = (Cerebrum.ObjectHandle)de.Key;
						using(Cerebrum.IComposite workgroup = workgroups.AttachConnector(wh))
						{
							Cerebrum.Management.Security.Reflection.WorkgroupDescriptor wgd = workgroup.Component as Cerebrum.Management.Security.Reflection.WorkgroupDescriptor;
							arrWorkgroups.Add(new Cerebrum.Specialized.StringAttributeEntry(wh, wgd.Name));

							using(Cerebrum.IContainer priveleges = wgd.GetSelectedPrivilegesVector())
							{
								if(priveleges!=null)
								{
									foreach(System.Collections.DictionaryEntry ve in (System.Collections.IEnumerable)priveleges)
									{
										Cerebrum.ObjectHandle vh = (Cerebrum.ObjectHandle)ve.Key;
										using(Cerebrum.IComposite privelege = priveleges.AttachConnector(vh))
										{
											Cerebrum.Management.Security.Reflection.PrivilegeDescriptor vgd = privelege.Component as Cerebrum.Management.Security.Reflection.PrivilegeDescriptor;
											if(!hshPriveleges.ContainsKey(vh))
											{
												hshPriveleges.Add(vh, vh);
												arrPriveleges.Add(new Cerebrum.Specialized.StringAttributeEntry(vh, vgd.Name));
											}
										}
									}
								}
							}
						}
					}
				}
			}
			using(Cerebrum.IContainer priveleges = prd.GetSelectedPrivilegesVector())
			{
				if(priveleges!=null)
				{
					foreach(System.Collections.DictionaryEntry ve in (System.Collections.IEnumerable)priveleges)
					{
						Cerebrum.ObjectHandle vh = (Cerebrum.ObjectHandle)ve.Key;
						using(Cerebrum.IComposite privelege = priveleges.AttachConnector(vh))
						{
							Cerebrum.Management.Security.Reflection.PrivilegeDescriptor vgd = privelege.Component as Cerebrum.Management.Security.Reflection.PrivilegeDescriptor;
							if(!hshPriveleges.ContainsKey(vh))
							{
								hshPriveleges.Add(vh, vh);
								arrPriveleges.Add(new Cerebrum.Specialized.StringAttributeEntry(vh, vgd.Name));
							}
						}
					}
				}
			}
			return new Cerebrum.Security.CerebrumPrincipal(cid, (Cerebrum.Specialized.StringAttributeEntry[])arrWorkgroups.ToArray(typeof(Cerebrum.Specialized.StringAttributeEntry)), (Cerebrum.Specialized.StringAttributeEntry[])arrPriveleges.ToArray(typeof(Cerebrum.Specialized.StringAttributeEntry)));
		}
		public static Cerebrum.Security.CerebrumPrincipal Authenticate(Cerebrum.Integrator.DomainContext context, string name, string password)
		{
			using(Cerebrum.Data.TableView principals = GetView(context, "Principals"))
			{
				if(principals==null) 
				{
					return null;
				}
				Cerebrum.ObjectHandle prh = FindHandle(principals, principals.GetItemProperties(null)["Name"], name);
				if(prh==Cerebrum.ObjectHandle.Null)
				{
					return null;
				}
				Cerebrum.Data.ComponentItemView item = principals.CreateItemViewByObjectHandle(prh);
				using(Cerebrum.IComposite principal = item.GetComposite())
				{
					if(principal==null) 
					{
						return null;
					}
					Cerebrum.Management.Security.Reflection.PrincipalDescriptor prd = principal.Component as Cerebrum.Management.Security.Reflection.PrincipalDescriptor;
					if(prd==null)
					{
						return null;
					}
					if(prd.Password != password)
					{
						return null;
					}

					return LoadPrincipal(prh, prd, true);
				}
			}
		}
		public static Cerebrum.Security.CerebrumPrincipal LoadPrincipal(Cerebrum.Integrator.DomainContext context, Cerebrum.ObjectHandle principalHandle, bool isAuthenticated)
		{
			if(principalHandle==Cerebrum.ObjectHandle.Null)
			{
				return null;
			}
			using(Cerebrum.Data.TableView principals = GetView(context, "Principals"))
			{
				if(principals==null) 
				{
					return null;
				}

				Cerebrum.Data.ComponentItemView item = principals.CreateItemViewByObjectHandle(principalHandle);
				using(Cerebrum.IComposite principal = item.GetComposite())
				{
					if(principal==null) 
					{
						return null;
					}
					Cerebrum.Management.Security.Reflection.PrincipalDescriptor prd = principal.Component as Cerebrum.Management.Security.Reflection.PrincipalDescriptor;
					if(prd==null)
					{
						return null;
					}

					return LoadPrincipal(principalHandle, prd, isAuthenticated);
				}
			}
		}

        public class VectorLateBinding
        {
            public string TableName;
            public string FieldName;
            public Cerebrum.ObjectHandle ComponentHandle;
            public Cerebrum.ObjectHandle AttributeHandle;
        }
    }
}

